;; rcv-to-bnd.scm

(use base64)

(module boundary (rcv-bnd-buffer-sz
				  rcv-to-bnd
				  get-header-str
				  clear-to-bnd)
  (import scheme chicken srfi-1 srfi-13 extras utils base64)
  (import common mime-db)

(define rcv-bnd-buffer-sz (mk-glob-prm 4096))
  
(define (str->crlf str)
  (let crlf-lp ((eolls '("\r\n" "\n" "\r")))
    (if (null? eolls)
      #f
      (let* ((eol (car eolls))
             (pos (string-contains str eol 0 
                                  (min (string-length str) 200))))
        (if pos
          (if (string=? eol "\n")
            (if (eqv? (string-ref str (- pos 1)) #\return)
              "\r\n\r\n"
              "\n\n")
            "\r\r")
            (crlf-lp (cdr eolls)))))))

(define (lines-to-bnd inport bnd initstr outport outproc)
  (let ((initstr-port (open-input-string initstr))
        (outstr-port (open-output-string)))
    (let ln-bnd-lp ((instr ""))
      (let* ((instr-ln (if (not (eqv? instr #!eof))
                         (read-line initstr-port)
                         #!eof))
             (ln (if (eqv? instr-ln #!eof)
                   (read-line inport)
                   instr-ln)))
        (cond
          ((eqv? ln #!eof)
           (values (get-output-string outstr-port) "" 'eof))

          ((string-contains ln bnd)
           ;; if there is data left in initstr-port we extract the data
           ;; and return as next.  Otherwise no next to return.
           (let* ((next-data (read-all initstr-port))
                  (next (if (eqv? next-data #!eof)
                          ""
                          next-data))
                  (lines (get-output-string outstr-port)))
           (if (string-suffix? "--" ln)
             (values lines next 'final)
             (values lines next 'more))))

          (else
            (let* ((ln+ (if outproc
                          (outproc ln)
                          ln)))
              (if outport 
                  (write-line ln+ outport)
                  (write-line ln+ outstr-port))
              (ln-bnd-lp instr-ln))))))))
         
(define (bin-to-bnd inport bnd init chk-final outport)
  (let ((bndlen (string-length bnd))
        (kmp-vec (make-kmp-restart-vector bnd)))
    (let bin-bnd-lp ((i 0) 
                     (buflen (- (rcv-bnd-buffer-sz) 
                                (string-length init)))
                     (rcvd "") 
                     (mstr init))
      (let ((s (if (> buflen 0)
                 (string-append mstr (read-string buflen inport))
                 mstr)))
        (if (eqv? s #!eof)
          (values (string-append rcvd mstr) "" 'eof)
          (let ((j (string-kmp-partial-search bnd kmp-vec s i)))
            (cond
              ((< j 0)
               (let ((bpos (- (- j) bndlen)))
                 ;; bpos > 0 -> beginning of bnd in s
                 ;; bpos < 0 -> start of bnd is at end of rcvd
                 (let* ((outres 
                         (cond
                          ((> bpos 0)
                           (string-append rcvd (string-take s bpos)))

                          (else
                           ;; remove -j char from end of rcvd
                           (if (string-null? rcvd)
                              ""
                              (string-drop-right rcvd (- j))))))
                        
                        (nextpart (string-drop s (- j)))
                        (final (and chk-final 
                                    (string-prefix? "--" nextpart)))
                        (savepart 
                          (string-trim nextpart
                                       (lambda (chr)
                                         (if final        
                                           (case chr
                                             ((#\- #\newline #\return)
                                               #t)
                                             (else #f))
                                           (case chr
                                             ((#\newline #\return) #t)
                                             (else #f)))))))
                  ;; finished
                  (values (if outport
                            (begin
                              (write-string outres #f outport)
                              "")
                            outres)
                          savepart (if final
                                      'final
                                      'more)))))

              ((> j 0)
               ;; some char match at end of s, 
               ;;    loop to next read-string
               ;; save non-matching characters in rcvd
               ;;    and matching chars in mstr
               ;; substract len of mstr from buflen 
               (let ((m (string-take-right s j))
                     (rcv (string-drop-right s j)))
                 (bin-bnd-lp j 
                             (- (rcv-bnd-buffer-sz) (string-length m))
                             (if outport
                                (begin
                                  (write-string rcv #f outport)
                                  "")
                                (string-append rcvd rcv))
                             m)))

              (else
                ;; j is 0
                (if (string-null? s)
                  ;; nothing left to read, say we're done.
                  (values (if outport
                            (begin
                              (write-string mstr #f outport)
                              "")
                            (string-append rcvd mstr)) "" 'done)
                  
                  (bin-bnd-lp j 
                              (rcv-bnd-buffer-sz) 
                              (if outport
                                (begin
                                  (write-string s #f outport)
                                  "")
                                (string-append rcvd s)) ""))))))))))
                
(define (rcv-to-bnd inport bnd mime-t init #!key (encoding "") 
                                                 (chk-final #t)
                                                 outport 
                                                 outproc)
  (if (text-mime-type? mime-t encoding)
    (lines-to-bnd inport bnd init outport outproc)
    (bin-to-bnd inport bnd init chk-final outport)))

(define (get-header-str inport initstr)
  (let* ((srchstr (if (< (string-length initstr) 200)
                    (string-append initstr 
                                   (read-string 
                                     (- 200 (string-length initstr)) 
                                     inport))
                    initstr))
         (eol (str->crlf srchstr)))
    (if (not eol)
      (values "" srchstr 'more)
      (let-values (((res next final) (rcv-to-bnd inport 
                                                 eol "not-txt"
                                                 srchstr 
                                                 chk-final: #f)))
        (values res next final)))))

(define (clear-to-bnd inport bnd initstr)
  (let-values (((res next final) (rcv-to-bnd inport bnd "not-txt"
                                                    initstr)))
    ;; since we aren't interested in res, just return ""
    ;; but we do want to remove leading eol chars at front of next
    (values "" 
            (string-trim next (lambda (chr)
                                (case chr
                                  ((#\newline #\return) #t)
                                  (else #f))))
            final)))
                                                    
)
