;; mime-db.scm

(module mime-db (extension->mime-type
				 filepath->mime-type
				 accept-str->mime-type
				 text-mime-type?)

		(import scheme chicken files data-structures srfi-1 srfi-13)
		(import common)

;; mime.ndx contains /globs/ info parsed into an alist with 
;; extension names as keys.  Keys originally having a leading
;; "." the mime-db alist with dot stripped.  Those keys that
;; didn't have a leading dot are in the "dotless" list.
;; (define mime-db (("<key>" . "<mime-type>") ...))
(include "mime.ndx.scm")

(define (extension->mime-type key)
  (let ((mime-t (and key (k->vci key mime-db))))
	  (or mime-t "application/octet-stream")))

(define (filepath->mime-type fpath)
  (and fpath
	   (let ((ext (pathname-extension fpath)))
		 (cond
		  (ext
		    (extension->mime-type ext))
		  (else
		    (let ((filenm (pathname-file fpath)))
			  (or (and filenm (k->vci filenm dotless-mime-db))
				  "application/octet-stream")))))))

(define matcher-txt (irx-search '(: ($ (w/nocase (or "/html" "/xml" 
											  "/xhtml" "/plain"))))))

(define matcher-filetype (irx-search '(: ($ (w/nocase (or "image/png" "image/jpg"
											  "image/gif" "image/tif"
											  "application/pdf"))))))

;; accept string header--http
(define (accept-str->mime-type accept-str)
  (let ((m (matcher-txt accept-str 1)))
	(or (and m (string-append "text" m))
		(matcher-filetype accept-str 1))))

(define (text-mime-type? mime-t/ext #!optional (encoding ""))
  (let ((mtype (or (and (string-contains mime-t/ext "/") mime-t/ext)
                   (extension->mime-type mime-t/ext))))
    (or (not (boolean? (string-contains-ci mtype "text")))
        (and (not (boolean? encoding))
             (not (boolean? (string-contains-ci encoding "base64")))))))

)
